%% dpdt.m: Fit sol'n to PDE with linear coefficients to BRFSS data
% 
%
% Inputs:
%   - T0: years
%   - hist: BRFSS histograms
%   - Xin: BRFSS survey data
%
% Outputs:
%   - 'BRFSS_time.avi': BRFSS dist'n over time (empirical, fit, and PDE
%   fit)
%   - 'Summary.pdf': Fitted BRFSS parameters over time

function dpdt(T0, hist, Xin)
global x xx X dx P P0 row col T Gparam D1 D2

% Years
T = T0;
Nt = length(T);

% BMIs by year
X = cell(Nt,1);

for t = 1:Nt
    X{t} = Xin(Xin(:,1)==T(t),2);
end

% Spatial resolution (bin edges) to make histogram
dx = 1;
x = floor(min(Xin(:,2)))-dx/2:dx:ceil(max(Xin(:,2)))+dx/2;
Nx = length(x)-1;

clearvars Xin

% Histograms
P = zeros(Nx, Nt);

for t = 1:Nt
    P(:,t) = histcounts(X{t}, x, 'Normalization', 'pdf');
end

x = 0.5*(x(1:end-1) + x(2:end));
xx = kron(x, ones(Nx,1));

% Compute PDF
% Smooth raw data using convolution
P0 = exp(-0.5*(-4:1:4).^2)/sqrt(2*pi);
P0 = conv(P(:,1),P0,'same');
P0 = P0/sum(P0*dx);

%% Derivative matrices
D1 = diag(ones(Nx-1,1),1);
D1 = sparse(D1 - D1')/2/dx;

D2 = -eye(Nx) + diag(ones(Nx-1,1),1);
D2 = sparse(D2 + D2')/dx^2;

%% Guess for parameters for solving PDE
% We fit parameters using a Taylor approximation (np = 2 implies linear)
%   xast(t) = xast(t0) + c1*(t-t0) + c2*(t-t0)^2 + ...
%   ki(t) = ki(t0) + c1*(t-t0) + c2*(t-t0)^2 + ...
%   kb(t) = kb(t0) + c1*(t-t0) + c2*(t-t0)^2 + ...
np = 2;

% param = [xast, ki, kb]
param = zeros(np,3,np-1);

% Initial guess for xast is linear regression of mean BMI vs year
y = zeros(Nt,1);

for t = 1:Nt
    y(t) = mean(X{t});
end

Xtemp = [T,ones(Nt,1)];
y = (Xtemp'*Xtemp)\(Xtemp'*y);

% y = regress(y,[T,ones(Nt,1)]);

param(1:2,1,1) = [y(1)*T(1)+y(2),y(1)];

% Initial uess for ki, kb is that they are constant, approx. = 0.1
param(1,2,1) = 0.08;
param(1,3,1) = 0.12;

%% Iteratively optimize parameters using MLE
n = 0;
dS = 0;

for k = 2:np
    if k > 2
        param(:,:,k-1) = param(:,:,k-2);
    end
    
    while n <= 2 || dS > 1e-6
        n = n+1;
        
        if n > 1
            S0 = S1;
        end
        
        % Optimize parameters one at a time
        for i = 1:3
            col = i;
            
            for j = 1:np
                row = j;
                Gparam = param(:,:,k-1);
                
                if i == 3 && j == np
                    [param(j,i,k-1), S1] = fminsearch(@obj, param(j,i,k-1));
                else
                    param(j,i,k-1) = fminsearch(@obj, param(j,i,k-1));
                end
            end
        end
        
        if n > 1
            dS = abs(S1-S0);
        end
    end
end

% save('dpdt.mat','param','h')
save('dpdt.mat')

%% Plot year-by-year
figure
vid = VideoWriter('BRFSS_time.avi');
open(vid)

Gparam = param(:,:,end);

[~, Ptemp] = ode45(@pdefun, T, P0);

for t = 1:Nt
    plot(x, Ptemp(t,:), '-b', 'LineWidth', 2)
    hold on; box on
    scatter(x, P(:,t), '.r')
    plot((x(1:end-1)+x(2:end))/2, hist(:,t), '--k')
    
    axis([10,60,0,0.12])
    
    xlabel('BMI','Interpreter','Latex')
    ylabel('PDF','Interpreter','Latex')
    title(strcat('\textbf{',num2str(T(t)),'}'),'Interpreter','Latex')
    
    %     saveas(gcf,strcat(num2str(T(t)),'.fig'))
%     save2pdf(num2str(T(t)),gcf,600)
    
    for i = 1:36
        frame = getframe(gcf);
        writeVideo(vid,frame);
    end
    hold off
end

% close
close(vid)

%% Plot parameters

figure
set(gcf, 'units', 'centimeters', 'pos', [1 1 16 4])

for i = 1:4
    subplot(1,4,i)
    hold on; box on;
    set(gca, 'units', 'centimeters', 'pos', [(i-1)*4+1 1 2.25 2.75])
    
    if i == 1
        ylabel('$x^*$', 'Interpreter', 'Latex', 'FontSize', 12)
        %         set(gca,'YTick', 24:2:30)
        %         set(gca,'YTickLabel',{'24','26','28','30'})
    elseif i == 2
        ylabel('$k_I$', 'Interpreter', 'Latex', 'FontSize', 12)
        %         set(gca,'YTick', 0.1:0.01:0.14)
        %         set(gca,'YTickLabel',{'0.1','0.11','0.12','0.13','0.14'})
    elseif i == 3
        ylabel('$\sqrt{k_b}\times 10^2$', 'Interpreter', 'Latex', 'FontSize', 12)
        %         set(gca,'YTick', 0.094:0.002:0.1)
        %         set(gca,'YTickLabel',{'9.4','9.6','9.8','10.0'})
    else
        ylabel('$k_I/k_b$', 'Interpreter', 'Latex', 'FontSize', 12)
        set(gca,'YTick', 10:5:20)
        %         set(gca,'YTickLabel',{'10','15','20'})
    end
    
    xlabel('Year ($t$)', 'Interpreter', 'Latex', 'FontSize', 12)
    
    if i <=3
        TS = T-T(1);
        TS = kron(TS', ones(np,1)).^kron((0:1:np-1)',ones(1,Nt));
        TS = TS'*param(:,i);
        
        if i == 1
            load('BMIresults.mat','BRpop')
            scatter(BRpop(:,1), BRpop(:,11), '.b')
        elseif i == 2
            yki = TS;
        elseif i == 3
            ykb = TS.^2;
        end
    else
        TS = yki./ykb;
        scatter(BRpop(:,1), BRpop(:,10), '.b')
    end
    
    plot(T, TS, '-b')
end

% save2pdf('Summary',gcf,600)
saveas(gcf,'Summary.fig')
close


end

%% Objective Function
function S = obj(p)
global x X row col T P0 Gparam % dx P

Gparam(row,col) = p;

% compute solution to PDE
[~, Ptemp] = ode45(@pdefun, T, P0);

% Ptemp = reshape(Ptemp',[],1)';
%
% Ptemp(Ptemp <= 0) = 1;
%
% S = -log(Ptemp)*reshape(P,[],1)*dx;

S = 0;
n = 0;

for t = 1:size(Ptemp,1)
    stemp = interp1(x,Ptemp(t,:),X{t});
    stemp(stemp <= 0) = [];
    n = n + length(stemp);
    S = S - sum(log( stemp ));
end

S = S/n;
end

%% Solve PDE
function dP = pdefun(t,P)
global x D1 D2 Gparam T
np = size(Gparam,1);

TS = t-T(1);
TS = TS * ones(1,np);
TS = TS.^(0:1:np-1);

xast = TS*Gparam(:,1);
ki = TS*Gparam(:,2);
kb = TS*Gparam(:,3);

a = ki*(xast - x');
b = kb*x';
dP = -D1*(a.*P) + 0.5*D2*(b.^2.*P);
end